@@ -21,11 +21,11 @@ use warnings;
 
 eval q{
     pop @INC if $INC[-1] eq '.';
-    use Net::FTP;
+
+    use Dpkg; # Dummy import to require the presence of Dpkg::*.
 };
 if ($@) {
-    warn "Please install the 'perl' package if you want to use the\n" .
-         "FTP access method of dselect.\n\n";
+    warn "Missing Dpkg modules required by the FTP access method.\n\n";
     exit 1;
 }
 
