@@ -52,9 +52,10 @@ our %EXPORT_TAGS = (
 );
 
 use Exporter qw(import);
+use Cwd;
 use File::Find;
 use File::Basename;
-use File::Path qw(make_path);
+use File::Path qw(make_path rmtree);
 use IPC::Cmd qw(can_run);
 use Test::More;
 
@@ -93,6 +94,7 @@ sub test_get_temp_path
     my $path = shift // _test_get_caller_dir();
     $path = 't.tmp/' . fileparse($path);
 
+    rmtree($path);
     make_path($path);
     return $path;
 }
@@ -111,45 +113,42 @@ sub test_get_perl_dirs
     if ($test_mode eq 'cpan') {
         return qw(t lib);
     } else {
-        return qw(t src/t lib utils/t scripts dselect);
+        return qw(t lib utils/t scripts dselect);
     }
 }
 
+sub _test_get_files
+{
+    my ($filter, $dirs) = @_;
+    my @files;
+    my $scan_files = sub {
+        push @files, $File::Find::name if m/$filter/;
+    };
+
+    find($scan_files, @{$dirs});
+
+    return @files;
+}
+
 sub all_po_files
 {
-    my $filter = shift // qr/\.(?:po|pot)$/;
-    my @files;
-    my $scan_po_files = sub {
-        push @files, $File::Find::name if m/$filter/;
-    };
-
-    find($scan_po_files, test_get_po_dirs());
-
-    return @files;
+    return _test_get_files(qr/\.(?:po|pot)$/, [ test_get_po_dirs() ]);
 }
 
 sub all_perl_files
 {
-    my $filter = shift // qr/\.(?:PL|pl|pm|t)$/;
-    my @files;
-    my $scan_perl_files = sub {
-        push @files, $File::Find::name if m/$filter/;
-    };
-
-    find($scan_perl_files, test_get_perl_dirs());
-
-    return @files;
+    return _test_get_files(qr/\.(?:PL|pl|pm|t)$/, [ test_get_perl_dirs() ]);
 }
 
 sub all_perl_modules
 {
-    return all_perl_files(qr/\.pm$/);
+    return _test_get_files(qr/\.pm$/, [ test_get_perl_dirs() ]);
 }
 
 sub test_needs_author
 {
-    if (not $ENV{DPKG_DEVEL_MODE} and not $ENV{AUTHOR_TESTING}) {
-        plan skip_all => 'developer test';
+    if (not $ENV{AUTHOR_TESTING}) {
+        plan skip_all => 'author test';
     }
 }
 
@@ -194,16 +193,17 @@ sub test_neutralize_checksums
     my $filename = shift;
     my $filenamenew = "$filename.new";
 
-    open my $fhnew, '>', $filenamenew or die;
-    open my $fh, '<', $filename or die;
+    my $cwd = getcwd();
+    open my $fhnew, '>', $filenamenew or die "cannot open new $filenamenew in $cwd: $!";
+    open my $fh, '<', $filename or die "cannot open $filename in $cwd: $!";
     while (<$fh>) {
         s/^ ([0-9a-f]{32,}) [1-9][0-9]* /q{ } . $1 =~ tr{0-9a-f}{0}r . q{ 0 }/e;
         print { $fhnew } $_;
     }
-    close $fh or die;
-    close $fhnew or die;
+    close $fh or die "cannot close $filename";
+    close $fhnew or die "cannot close $filenamenew";
 
-    rename $filenamenew, $filename or die;
+    rename $filenamenew, $filename or die "cannot rename $filenamenew to $filename";
 }
 
 1;
